/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package net.rmi.rjs.pk.LusCs;

import gui.ClosableJFrame;
import gui.layouts.DialogLayout;
import gui.run.RunButton;
import gui.run.RunCheckBox;

import javax.swing.*;
import java.awt.*;
import java.util.Observable;
import java.util.Observer;

/**
 * Copyright DocJava, inc. User: lyon
 * <p/>
 * Date: May 1, 2005
 * <p/>
 * Time: 5:54:44 PM
 */
public class CsRecordPanel extends JPanel implements Observer {
    private CsRecord csr;
    private JLabel speedLabel;
    private RunCheckBox busyCheckbox;
    private JLabel ageLabel;

    private void setBusy(boolean b) {
        csr.setBusy(b);
    }

    public CsRecordPanel(CsRecord csr) {
        this.csr = csr;
        speedLabel = new JLabel("      ");
        busyCheckbox = new RunCheckBox(csr.isBusy()) {
            public void run() {
                setBusy(this.isSelected());
            }


        };
        final long age = csr.getAge();

        ageLabel = new JLabel(age / 1000 + "");
        csr.addObserver(this);
        setLayout(new DialogLayout());
        add(new JLabel("speed in mflops:"));
        add(speedLabel);
        final String ipAddrString =
                csr.getIpAddress().getHostAddress();
        add(new JLabel("busy:"));
        add(busyCheckbox);
        add(new JLabel("age (sec)"));
        add(ageLabel);
        setBorder(BorderFactory.createTitledBorder(ipAddrString));
        update();
    }

    public void update() {
        speedLabel.setText(csr.getBmb().getMflops() + "");
        busyCheckbox.setSelected(csr.isBusy());
        ageLabel.setText(csr.getAge() + "");
        if (csr.isDead()) {
            setVisible(false);
            updateFrame();
        }
    }

    private void updateFrame() {
        Container parent = getParent();
        if (parent==null) return;
        parent.invalidate();
        JFrame frame = getFrame();
        if (frame == null) return;
        frame.pack();
    }

    public JFrame getFrame() {
        Container parent = getParent();
        if (parent == null) return null;

        while (!(parent instanceof JFrame))
            parent = parent.getParent();
        if (parent == null) return null;
        return (JFrame) parent;
    }

    public void update(Observable obs, Object obj) {
        update();
    }

    public static void main(String[] args) {
        CsRecords csrs = CsRecords.getTestRecords();
        displayCsRecords(csrs);
    }

    public static void displayCsRecords(CsRecords csrs) {
        ClosableJFrame cf = new ClosableJFrame();
        JPanel jp = new JPanel();
        Container c = cf.getContentPane();
        jp.setLayout(new FlowLayout());

        CsRecord csra[] = csrs.getRecords();
        for (int i = 0; i < csra.length; i++) {
            jp.add(new CsRecordPanel(csra[i]));
        }
        jp.add(new ButtonControlPanel(csrs));
        JScrollPane jsp = new JScrollPane(jp);
        c.add(jsp);
        cf.pack();
        cf.setVisible(true);
    }

}

class ButtonControlPanel extends JPanel {
    public ButtonControlPanel(final CsRecords csrs) {
        setLayout(new FlowLayout());
        add(new RunButton("terminate all") {
            public void run() {
                csrs.killAllServers();
            }
        });
        add(new RunButton("update benchmarks") {
            public void run() {
                csrs.updateBenchMarks();
                System.exit(0);
            }
        });
        add(new RunButton("quit") {
            public void run() {
                csrs.killAllServers();
                System.exit(0);
            }
        });
    }

}